# coding: utf-8

__title__ = 'scrapydwebx'
__version__ = '1.5.1'
__author__ = 'payala'
__author_email__ = 'ppayala@gmail.com'
__url__ = 'https://github.com/payala/scrapydwebx'
__license__ = 'GNU General Public License v3.0'
__description__ = ("Web app for Scrapyd cluster management, "
                   "with support for Scrapy log analysis & visualization.")
