  <div id="multinodes">
    <a class="link" href="{{ url_servers }}" target="_blank">To select nodes in the Servers page &raquo;</a>
    <div class="line-container">
      <div class="key"><span>*</span>multinodes</div>
      <input type="text" name="checked_amount" value="1" hidden />
      <div class="multiselect">
        <div class="selectBox" onclick="showCheckboxes();">
          <div class="value">
            <span id="selected_nodes_statement">{{ SCRAPYD_SERVERS[node-1] }}</span>
            <svg class="icon anchor" aria-hidden="true">
              <use xlink:href="#icon-down"></use>
            </svg>
          </div>
          <div class="overSelect"></div>
        </div>
        <div id="checkboxes" style="display: none;">
          <label for="checkCurrent"><input type="checkbox" id="checkCurrent" />Check current node only</label>
          <label for="syncFromServersPage"><input type="checkbox" id="syncFromServersPage" />Sync from Servers page</label>
          <label for="checkAll"><input type="checkbox" id="checkAll" />CheckAll / UncheckAll</label>
          <div id="nodes_checkboxes">
          {% for SCRAPYD_SERVER in SCRAPYD_SERVERS %}
            {% if SCRAPYD_SERVERS_GROUPS[loop.index0] and loop.changed(SCRAPYD_SERVERS_GROUPS[loop.index0]) %}
            <span> --- {{ SCRAPYD_SERVERS_GROUPS[loop.index0] }} --- </span>
            {% endif %}

            <label id="label_{{ loop.index }}" for="checkbox_{{ loop.index }}" {% if loop.index == node %}style="color: #409EFF; font-weight: 700;"{% endif %}>
              <input type="checkbox" id="checkbox_{{ loop.index }}" name="{{ loop.index }}"
              {% if loop.index in selected_nodes %}checked{% endif %} />[{{ loop.index }}] {{ SCRAPYD_SERVER }}
            </label>
          {% endfor %}
          </div>
        </div>
        <div class="form-item-error">Select at least one node</div>
      </div>
    </div>
  </div>
