        stats_loading_display: '',
        stats: '',
        stats_url: '',
        stats_class: '',
        stats_display: 'none',

        pages: '',
        items: '',
        pages_class: 'count_font count_safe',
        items_class: 'count_font count_safe',

        reason: '',
        reason_class: '',

        log_critical_count: '',
        log_error_count: '',
        log_warning_count: '',
        log_redirect_count: '',
        log_retry_count: '',
        log_ignore_count: '',

        log_critical_class: '',
        log_error_class: '',
        log_warning_class: '',
        log_redirect_class: '',
        log_retry_class: '',
        log_ignore_class: '',

        latest_item: '',
